//package adaptadorl4;

import java.awt.BorderLayout;
import javax.swing.border.EtchedBorder;
import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.awt.event.*;

/**
 * CGPanel. Esta clase contiene todos los  elementos necesarios para construir la 
 * interfaz de usuario para la aplicación.
 * En ésta se encuentran instancias de paneles especificos que se unen para
 * lograr un resultado final adecuado para el usuario.
 * @author Sabina Torres Figueroa
 * @author Pedro Duran Caneo
 * @author Jorge Pareja Ayala
 */
public class CGPanel extends JPanel
{

/**
 * Constructor por defecto. En éste, se crean las instancias necesarias de los
 * elementos que componen la interfaz gráfica del AdaptadorDeCS, como por
 * ejemplo el panel lateral, central e inferior. Además se crea la carta Smith 
 * sobre la cual se trabaja durante la simulación.
 */  
public CGPanel()
{
	ultimos = new ArrayList<JLabel>();
	canvitas=new Smith();
	canvitas.setSize(500, 500);
	menuBar();
	centro = new centralPanel(canvitas);
	fila = new buttomPanel(canvitas,"imgL4/greenpcb2.jpg","imgL4/carga.jpg");
	lastReal = lastImag = 0;
	columna = new leftPanel(canvitas,fila);
}

/**
 * Listener. Clase interna encargada de escuchar los eventos de la barra de menu.
 * Si voincide con alguno de los elementos que están dispuestos en la barra de
 * menu, agrega elementos en el panel inferior para dibujar el circuito
 * equivalente que se está simulando
 */
class Listener implements ActionListener
{

/**
 * actionPerformed. Método encargado de analizar los eventos ocurridos en los 
 * componentes gráficos del panel.
 * @param evento Objeto que se encarga de capturar el origen del evento.
 */
    public void actionPerformed(ActionEvent evento)
	{
		//Si se elige about en el menu
		if (evento.getSource() == About)
		    about();
		//Si se elige about en el menu
		if (evento.getSource() == Res)
		{
		    if( (lastReal = getValor("Ingrese Valor de Resistencia en [ohm]")) != -1)
		    {
		    	//fila.add(addComponent("Res.jpg"));
		    	lastImag=0;
		    	canvitas.setSerie(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/Res.jpg");
			lastLambda = 0;
			columna.zineq();
		    }
		}
		if (evento.getSource() == Cond)
		{
		    if( (lastReal = getValor("Ingrese Valor de Resistencia en [ohm]")) != -1)
		    	{//fila.add(addComponent("Cond.jpg"));
		    	lastImag=0;
	    		canvitas.setParalelo(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/Cond.jpg");
			lastLambda = 0;
			columna.zineq();
			}
		}
		if (evento.getSource() == rCap)
		{
		    if( (lastImag = getValor("Ingrese Valor de Reactancia en [ohm]")) != -1)
		    	{//fila.add(addComponent("rCap.jpg"));
		    	lastReal=0;
	    		canvitas.setSerie(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/rCap.jpg");
			lastLambda = 0;
			columna.zineq();
			}
		}
		if (evento.getSource() == sCap)
		{
		    if( (lastImag = getValor("Ingrese Valor de Reactancia en [ohm]")) != -1)
		    	{//fila.add(addComponent("sCap.jpg"));
		    	lastReal=0;
	    		canvitas.setParalelo(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/sCap.jpg");
			lastLambda = 0;
			columna.zineq();
			}
		}
		if (evento.getSource() == rInd)
		{
		    if( (lastImag = getValor("Ingrese Valor de Reactancia en [ohm]")) != -1)
		    	{//fila.add(addComponent("rInd.jpg"));
		    	lastReal=0;
	    		canvitas.setSerie(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/rInd.jpg");
			lastLambda = 0;
			columna.zineq();
			}
		}
		if (evento.getSource() == sInd)
		{
		    if( (lastImag = getValor("Ingrese Valor de Reactancia en [ohm]")) != -1)
		    	{//fila.add(addComponent("sInd.jpg"));
		    	lastReal=0;
	    		canvitas.setParalelo(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/sInd.jpg");
			lastLambda = 0;
			columna.zineq();
			}
		}
		if (evento.getSource() == Zeta)
		{
		    lastReal = getValor("Ingrese Valor de parte Real en [ohm]");
		    lastImag = getValor("Ingrese Valor de parte Imaginaria en [ohm]");
	    		canvitas.setSerie(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/Zeta.jpg");
			lastLambda = 0;
			columna.zineq();
		}
		if (evento.getSource() == Admi)
		{
		    lastReal = getValor("Ingrese Valor de parte Real en [ohm]");
		    lastImag = getValor("Ingrese Valor de parte Imaginaria en [ohm]");
		    //fila.add(addComponent("Admi.jpg"));
		    	//Error();
	    		canvitas.setParalelo(lastReal,lastImag,columna.Zo);
			fila.addComponent("imgL4/Admi.jpg");
			lastLambda = 0;
			columna.zineq();
		
		}
		if (evento.getSource() == Line)
		{
		    if( (lastLambda = getValor("Ingrese largo de la linea en [lambdas]")) != -1)
		    	{//fila.add(addComponent("Line.jpg"));
		    	canvitas.setLinea(lastLambda,columna.Zo);
			fila.addComponent("imgL4/Line.jpg");
			lastReal = lastImag = 0;
			columna.zineq();
			}
		}
	fila.validate();
	canvitas.repaint((long)100);
	}
 

}

/**
 * Método principal. En éste, se crean las instancias necesarias de los
 * elementos que componen la interfaz gráfica del AdaptadorDeCS, como por
 * ejemplo el panel lateral, central e inferior. Además se crea la carta Smith 
 * sobre la cual se trabaja durante la simulación.
 */
  public void run()
  {

    BorderLayout border = new BorderLayout(); // Create a layout manager
    setLayout(border); // Set the container layout mgr
    EtchedBorder edge = new EtchedBorder(EtchedBorder.RAISED); // Button border

    add(columna, BorderLayout.WEST);
    columna.setBorder(edge);

    add(barra, BorderLayout.NORTH);
    barra.setBorder(edge);

    add(fila, BorderLayout.SOUTH);
    fila.setBorder(edge);

    add(centro, BorderLayout.CENTER);
    centro.setBorder(edge);

  }
  
/**
 * Método privado que crea y organiza la barra de menú del applet con la cual
 * posteriormente se agregarán componentes a ejercicio que se está
 * desarrollando.
 */
private void menuBar()
{
	//Creacion de la barra de menu
	barra	= new JMenuBar();
	barra.setBackground(Color.CYAN);
	Add	= new JMenu("Agregar");
	Add.setFont(new Font("Serif", Font.BOLD, 16));
	Help	= new JMenu("Ayuda");
	Help.setFont(new Font("Serif", Font.BOLD, 16));
	Edit	= new JMenu("Editar");
	Edit.setFont(new Font("Serif", Font.BOLD, 16));
	
	Zeta	= new JMenuItem("Impedancia Serie");
	rCap	= new JMenuItem("Condensador Serie");
	rInd	= new JMenuItem("Inductor Serie");
	Res	= new JMenuItem("Resistencia Serie");

	Admi	= new JMenuItem("Impedancia Paralela");
	sCap	= new JMenuItem("Condensador Paralelo");
	sInd	= new JMenuItem("Inductor Paralelo");
	Cond	= new JMenuItem("Resistencia Paralela");
	
	Line	= new JMenuItem("Linea de Tx");
	//Trafo	= new JMenuItem("Transformador Lambda/4");
	//Undo	= new JMenuItem("Deshacer");
	//Delete	= new JMenuItem("Eliminar");
	About	= new JMenuItem("About");
	
	
	//Agregar Items al menu
	//Add.add(sub1 = new JMenu("Elementos en Serie"));
	Add.add(Zeta);
	Add.add(rCap);
	Add.add(rInd);
	Add.add(Res);
	//Add.add(sub2 = new JMenu("Elementos en Paralelo"));
	Add.add(Admi);
	Add.add(sCap);
	Add.add(sInd);
	Add.add(Cond);
	
	Add.add(Line);
	//Add.add(Trafo);
	
	//Edit.add(Undo);
	//Edit.add(Delete);
	
	Help.add(About);
	
	barra.add(Add);
	//barra.add(Edit);
	barra.add(Help);
		
	//Listener para el menu
	Listener Flistener = new Listener();
	About.addActionListener(Flistener);
	Zeta.addActionListener(Flistener);
	rCap.addActionListener(Flistener);
	rInd.addActionListener(Flistener);
	Res.addActionListener(Flistener);
	Admi.addActionListener(Flistener);
	sCap.addActionListener(Flistener);
	sInd.addActionListener(Flistener);
	Cond.addActionListener(Flistener);
	Line.addActionListener(Flistener);
	//Undo.addActionListener(Flistener);
	//Adaptar.addActionListener(Flistener);
	//zBut.addActionListener(Flistener);
	//fBut.addActionListener(Flistener);
	
	//Delete.addActionListener(Flistener);
	//barra.repaint((long)100);

}

/**
 * Método privado que despliega una vantana de dialogo para dar los datos de
 * los autores del applet.
 */
private void about()
{
	String st = new String();
	st+="Elo329, Diseno y Programacion Orientados a Objetos\n";
	st+="    Integrantes del Grupo:\n";
	st+="       - Sabina Torres Figueroa  2104128-9\n";
	st+="       - Pedro Duran Caneo        2504032-5\n";
	st+="       - Jorge Pareja Ayala          2491011-3\n";
	JOptionPane.showOptionDialog(this,st,"Acerda de...",JOptionPane.DEFAULT_OPTION,JOptionPane.INFORMATION_MESSAGE,null,null,null);//new ImageIcon("csmith.gif")
}

/**
 * Método privado que despliega una vantana de dialogo informando que se ha
 * ingresado datos erroneos y que no se pude continuar.
 */
private void Error()
{
	String st = new String();
	st+="Para un buen funcionamiento del programa\n";
	st+="debe ingresar los siguientes datos:\n";
	st+="       - Frecuencia de Operación\n";
	st+="       - Impedancia caracteristica de la linea\n";
        st+="       - Epsilon relativo del dielectrico\n";
	st+="       - Impedancia de carga\n";
	JOptionPane.showOptionDialog(this,st,"Error!!",JOptionPane.DEFAULT_OPTION,JOptionPane.INFORMATION_MESSAGE,null,null,null);//new ImageIcon("csmith.gif")
}


/**
 * Método privado que analiza los datos ingresados mediante los JTextField
 * para ello se utiliza el método estático parseDouble de la clase Double.
 * El modo de uso del método es ingresando el valor en ASCII de la cantidad a
 * analizar y el método retorna su valor como punto flotante.
 * @param st String que contiene el valor ASCII que se desea analizar.
 * @return Retorna el valor ingresado por st en formato de punto flotante.
 */
private double getValor(String st)
{
	String data = JOptionPane.showInputDialog(null,st,JOptionPane.QUESTION_MESSAGE);
	if(data != null)
	{
		return Double.parseDouble(data);
	}
	else return -1;
}


  JMenuBar	barra;
  JMenu		Add, Help, Edit;
  JMenu		sub1, sub2;
  JMenuItem	Zeta, rCap, rInd, Res;
  JMenuItem	Admi, sCap, sInd, Cond;
  JMenuItem	Line, Trafo, Undo, Delete, About;
  
 // JPanel	columna;//, centro, fila;

  ArrayList	<JLabel> ultimos;
  Smith		canvitas;
  buttomPanel	fila;
  centralPanel	centro;
  leftPanel	columna;
  int		last;
  double	lastReal, lastImag, lastLambda;

}
