//package aprendiendo;

import java.awt.*;

/**
 * Clase que dibuja una Carta Smith con sus círculos de resistencia cte. y sus círculos reactancia cte. Se puede agregar
 * trozos de líneas, Impedancias en serie, Impedancia en paralelo, las cuales se suman (dependiendo el elemento) a la impedancia vista en la línea. 
 * En el momento en que se agrega algún elemento, se dibuja el circulo de ROEV cte, el círculo de resistencia cte y de reactancia cte de 
 * la Impedancia total vista por la línea.
 * @author Sabina Torres Figueroa
 * @author Pedro Duran Caneo
 * @author Jorge Pareja Ayala
 */

public class Smith extends Canvas {

      private double entre=0.008;
      /**
       * Impedancia equivalente
       */
      private Impedancia Zequivalente=new Impedancia();
      private  boolean dibuja=false;
      private double Z0;
      private double r1n,x1n;
      private double lambdaOperacion;

      //radio
         private int radio;
      // Resistencia normalizada
         private double r;
     //reactancia normalizada
         private double xi;
      //anchura y altura del canvas
         private int anchoCanvas, altoCanvas;
      //origenes
         private int orgY, orgX, orgY2, orgX2;

                 /**
                  * Constructor. Inicializa con color blanco el fondo del canvas, 
                  * e inicializa el lambda de operación de la Carta Smith a 3 metros.
                  */
                     public Smith( ) {
                           this.lambdaOperacion=3;
                           setBackground(Color.white);
                     }

                /**
                 * Método público que permite sumar una impedancia en serie a la impedancia vista por la 
                 * línea, la Impedancia resultante se normaliza y se dibuja el círculo de resistencia cte., 
                 * el círculo de reactancia cte. y el círculo de ROEV cte.
                 * @param real Parte real de la impedancia que se suma en serie a la impedancia vista por la línea.
                 * @param compleja Parte imaginaria de la impedancia que se suma en serie a la Impedancia vista por la línea.
                 * @param z Impedancia característica de la línea.
                 */

                     public void setSerie(double real, double compleja, double z){
                         Zequivalente.serie(real, compleja);
                         Z0=z;
                         r1n=Zequivalente.R/Z0;
                         x1n=Zequivalente.X/Z0;
                         dibuja=true;
                         repaint();
                     }
               /**
                * Método público que permite sumar una impedancia en paralelo a la impedancia vista por la
                * línea, la Impedancia resultante se normaliza y se dibuja el círculo de resistencia cte.,
                * el círculo de reactancia cte. y el círculo de ROEV cte.
                * @param real Parte real de la impedancia que se suma en paralelo a la impedancia vista por la línea.
                * @param compleja Parte imaginaria de la impedancia que se suma en paralelo a la Impedancia vista por la línea.
                * @param z Impedancia característica de la línea.
                */

                     public void setParalelo(double real, double compleja, double z){
                        Zequivalente.paralelo(real, compleja);
                        Z0=z;
                        r1n=Zequivalente.R/Z0;
                        x1n=Zequivalente.X/Z0;
                        dibuja=true;
                        repaint();
                     }

               /**
                * Método público que establece el lambda de operación de la Carta Smith
                * @param lamda Lambda a la que opera la línea de transmisión.
                */

                    public void setlambdaOperacion(double lamda){
                        lambdaOperacion=lamda;
                    }
               
                              /**
                               * Método público que permite insertar un trozo de línea de transmisión de un largo y
                               * impedancia característica bien definidos
                               * @param largolambdas Largo del trozo de línea en número de lambdas (de la línea de trasmisión)
                               * @param z0 Impedancia característica del trozo de línea a insertar
                               */


                    public void setLinea(double largolambdas, double z0){
                              double x=Math.tan(2*Math.PI*largolambdas);
                              double al=Zequivalente.R;
                              double bl=Zequivalente.X;
                              double denominador=(al*al+bl*bl)*x*x-2*Z0*bl*x+z0*z0;
                              double numeradorRe;
                              double numeradorIm;

                             if(Math.cos(2*Math.PI*largolambdas)==0.0)   {
                                   numeradorRe=z0*z0*al;
                                   numeradorIm=-z0*z0*bl;
                                   denominador=(al*al+bl*bl);
                    
                              }
                             else{
                                  denominador=(al*al+bl*bl)*x*x-2*Z0*bl*x+z0*z0;
                                  numeradorRe=z0*z0*al*(x*x+1);
                                  numeradorIm=-z0*(z0*bl*x*x-(z0*z0-al*al-bl*bl)*x-z0*bl);
                             }
                   
                                 Zequivalente.R=numeradorRe/denominador;
                                 Zequivalente.X=numeradorIm/denominador;
                                 r1n=Zequivalente.R/Z0;
                                 x1n=Zequivalente.X/Z0;
                                 dibuja=true;
                                 repaint();
                    }


                           /**
                            * Regresa la parte real de la impedancia equivalente vista por la linea
                            * @return Parte resistiva de la Impedancia vista por la línea de Transmisión
                            */

                               public double getr(){
                                    return Zequivalente.R;
                               }
                           /**
                            * Regresa la parte imaginaria de la impedancia equivalente vista por la linea
                            * @return Parte imaginaria de la Impedancia vista por la línea de Transmisión
                            */
                              public double getx(){
                                    return Zequivalente.X;
                              }

                          /**
                           * Método público encargado de dibujar todos los elementos visibles
                           * @param g Referencia de la clase Graphics
                           */

                  public void paint(Graphics g){
                             Graphics2D g2d = (Graphics2D) g;
                             anchoCanvas=getSize().width;
                             altoCanvas=getSize().height;
                             FontMetrics fm=g2d.getFontMetrics();
                             int charAncho=fm.stringWidth("0");
                             radio=anchoCanvas/2-2*charAncho;
                           //origenes
                             orgX=anchoCanvas/2;
                             orgY=altoCanvas/2;
           
                
                            g2d.setColor(Color.DARK_GRAY);
                            g2d.drawOval(orgX-radio, orgY-radio, 2*radio, 2*radio);
                            g2d.setColor(Color.LIGHT_GRAY);
               
                            rCirculos( g2d );  // dibuja los circulos de resistencia costante
                            xCirculos( g2d );  // dibuja los circulos de reactancia constante
                            g2d.setColor(Color.red);
               
                               if(dibuja==true)  {
                                      marcainterseccion(g2d) ;
                                      g2d.setColor(Color.green);
                                      marcacirculo(g2d);
                               }
                  }

            /**
             * Método privado que dibuja el círculo de resistencia cte. y el círculo de reactancia cte. de la impedancia vista por la línea
             * @param g2d Referencia de la clase Graphics
             */
            private void marcainterseccion(Graphics2D g2d) {

                          double escalaX=(double)radio;
                          double  escalaY=(double)radio;
                          double min=(r1n-1)/(1+r1n), max=1;
                          int  x1=(int)(orgX+min*escalaX);int y1=orgY;
                          int x2, y2;
                    for(double v=min; v<=max; v=v+entre){
                          y2=orgY-(int)(resistencia(v, r1n)*escalaY);
                          x2=orgX+(int)(v*escalaX);
                          g2d.drawLine(x1, y1, x2, y2);
                          x1=x2; y1=y2;
                     }


                         x1=(int)(orgX+min*escalaX);y1=orgY;
                    for(double v=min; v<=max; v=v+entre){
                         y2=orgY+(int)(resistencia(v, r1n)*escalaY);
                         x2=orgX+(int)(v*escalaX);
                         g2d.drawLine(x1, y1, x2, y2);
                         x1=x2; y1=y2;
                    }

                       
                if(x1n==0){

                             min=-1;max=1;
                          escalaX=(double)radio;
                          escalaY=(double)radio/(x1n*reactancia(min,x1n));
                          x1=(int)(orgX+min*escalaX);
                          y1=(int)orgY;

                              for(double v=min; v<=max; v=v+entre){
                                 y2=orgY;
                                 x2=orgX+(int)(v*escalaX);
                                 g2d.drawLine(x1, y1, x2, y2);
                                 x1=x2; y1=y2;
                              }

                }

                         
               else if(Math.abs(x1n)>=1){


                       if(x1n>0){
                          min=(x1n-1)/x1n;max=1;
                          escalaX=(double)radio;
                          escalaY=(double)radio/(x1n*reactancia(min,x1n));
                          x1=(int)(orgX+min*escalaX);
                          y1=(int)(orgY-reactancia(min,x1n)*escalaY);

                              for(double v=min; v<=max; v=v+entre){
                                 y2=orgY-(int)(reactancia(v,x1n)*escalaY);
                                 x2=orgX+(int)(v*escalaX);
                                 g2d.drawLine(x1, y1, x2, y2);
                                 x1=x2; y1=y2;
                              }


                      //1 a 2

                           max=(x1n*x1n-1)/(x1n*x1n+1);
                           escalaY=(double)radio/(x1n*reactancia2(min,x1n));
                           x1=(int)(orgX+min*escalaX);
                           y1=(int)(orgY-reactancia2(min,x1n)*escalaY);
                               for(double v=min; v<=max; v=v+entre){
                                  y2=orgY-(int)(reactancia2(v,x1n)*escalaY);
                                  x2=orgX+(int)(v*escalaX);
                                  g2d.drawLine(x1, y1, x2, y2);
                                  x1=x2; y1=y2;
                              }


                       }
                       else{

                                          //-1 a -2

                                  min=(x1n+1)/x1n;max=(x1n*x1n-1)/(x1n*x1n+1);
                                  escalaY=(double)radio/(x1n*reactancia(min,x1n));
                                  x1=(int)(orgX+min*escalaX);
                                  y1=(int)(orgY-reactancia(min,x1n)*escalaY);

                                for(double v=min; v<=max; v=v+entre){
                                   y2=orgY-(int)(reactancia(v,x1n)*escalaY);
                                   x2=orgX+(int)(v*escalaX);
                                   g2d.drawLine(x1, y1, x2, y2);
                                   x1=x2; y1=y2;
                                }


                                      //-1 a -2

                                   min=(x1n+1)/x1n; max=1;
                                   escalaY=(double)radio/(x1n*reactancia2(min,x1n));
                                   x1=(int)(orgX+min*escalaX);y1=(int)(orgY-reactancia2(min,x1n)*escalaY);
                                   for(double v=min; v<=max; v=v+entre){
                                  y2=orgY-(int)(reactancia2(v,x1n)*escalaY);
                                  x2=orgX+(int)(v*escalaX);
                                  g2d.drawLine(x1, y1, x2, y2);
                                  x1=x2; y1=y2;
                              }
                                

                       }


                   }


                   else{

                        if(x1n>0){

                                min=(x1n*x1n-1)/(x1n*x1n+1);max=1;
                                escalaX=(double)radio;
                                escalaY=(double)radio;
                                x1=(int)(orgX+min*escalaX); y1=(int)(orgY-reactancia(min,x1n)*escalaY);


                            for(double v=min; v<=max; v=v+entre){
                                  y2=orgY-(int)(reactancia(v,x1n)*escalaY);
                                  x2=orgX+(int)(v*escalaX);
                                  g2d.drawLine(x1, y1, x2, y2);
                                  x1=x2; y1=y2;
                            }

                        
                       }
                       else{
                               min=(x1n*x1n-1)/(x1n*x1n+1);max=1;
                               escalaX=(double)radio;
                               escalaY=(double)radio;
                               x1=(int)(orgX+min*escalaX); y1=(int)(orgY-reactancia(min,x1n)*escalaY);

                                x1=(int)(orgX+min*escalaX);y1=(int)(orgY-reactancia2(min,x1n)*escalaY);


                                 for(double v=min; v<=max; v=v+entre){
                                      y2=orgY-(int)(reactancia2(v,x1n)*escalaY);
                                      x2=orgX+(int)(v*escalaX);
                                      g2d.drawLine(x1, y1, x2, y2);
                                      x1=x2; y1=y2;
                                  }

                       }


                   }


                }

      /**
       * Método privado que dibuja los círculos de resistencia cte. de la Carta Smith
       * @param g2d Referencia de la clase Graphics
       */
       private void rCirculos(Graphics2D g2d ){
          for( int i=0;i<80;i++){
                          double valor[]=rValores();
                          double escalaX=(double)radio;
                          double  escalaY=(double)radio;
                          r=valor[i];
                          double min=(r-1)/(1+r), max=1;
                          int  x1=(int)(orgX+min*escalaX);int y1=orgY;
                          int x2, y2;
                    for(double v=min; v<=max; v=v+entre){
                          y2=orgY-(int)(resistencia(v)*escalaY);
                          x2=orgX+(int)(v*escalaX);
                          g2d.drawLine(x1, y1, x2, y2);
                          x1=x2; y1=y2;
                     }


                            x1=(int)(orgX+min*escalaX);y1=orgY;
                    for(double v=min; v<=max; v=v+entre){
                         y2=orgY+(int)(resistencia(v)*escalaY);
                         x2=orgX+(int)(v*escalaX);
                         g2d.drawLine(x1, y1, x2, y2);
                         x1=x2; y1=y2;
                    }
            }



       }


        /**
       * Método privado que dibuja los círculos de reactancia cte. de la Carta Smith
       * @param g2d Referencia de la clase Graphics
       */
             private void xCirculos(Graphics2D g2d ){


                    double valorx2[]=xValores2();
                for(int indice=0;indice<35;indice++){
                    xi=valorx2[indice];

                          double min=(xi-1)/xi,max=1;
                          double escalaX=(double)radio;
                          double escalaY=(double)radio/(xi*reactancia(min));
                          int x1=(int)(orgX+min*escalaX);int y1=(int)(orgY-reactancia(min)*escalaY);
                          int x2,y2;
                              for(double v=min; v<=max; v=v+entre){
                                 y2=orgY-(int)(reactancia(v)*escalaY);
                                 x2=orgX+(int)(v*escalaX);
                                 g2d.drawLine(x1, y1, x2, y2);
                                 x1=x2; y1=y2;
                              }
                   

                      //1 a 2

                           max=(xi*xi-1)/(xi*xi+1);
                           escalaY=(double)radio/(xi*reactancia2(min));
                           x1=(int)(orgX+min*escalaX);
                           y1=(int)(orgY-reactancia2(min)*escalaY);
                               for(double v=min; v<=max; v=v+entre){
                                  y2=orgY-(int)(reactancia2(v)*escalaY);
                                  x2=orgX+(int)(v*escalaX);
                                  g2d.drawLine(x1, y1, x2, y2);
                                  x1=x2; y1=y2;
                              }
                    
                      xi=-valorx2[indice];

                            //-1 a -2

                            min=(xi+1)/xi;max=(xi*xi-1)/(xi*xi+1);
                            escalaY=(double)radio/(xi*reactancia(min));
                            x1=(int)(orgX+min*escalaX);
                            y1=(int)(orgY-reactancia(min)*escalaY);
                                    
                                for(double v=min; v<=max; v=v+entre){
                                   y2=orgY-(int)(reactancia(v)*escalaY);
                                   x2=orgX+(int)(v*escalaX);
                                   g2d.drawLine(x1, y1, x2, y2);
                                   x1=x2; y1=y2;
                                }
                           

                     //-1 a -2

                           min=(xi+1)/xi; max=1;
                           escalaY=(double)radio/(xi*reactancia2(min));
                           x1=(int)(orgX+min*escalaX);y1=(int)(orgY-reactancia2(min)*escalaY);
                               for(double v=min; v<=max; v=v+entre){
                                  y2=orgY-(int)(reactancia2(v)*escalaY);
                                  x2=orgX+(int)(v*escalaX);
                                  g2d.drawLine(x1, y1, x2, y2);
                                  x1=x2; y1=y2;
                              }
                }
                          double valorx1[]=xValores1();
                    for(int i=0;i<45;i++ ){
                               xi=valorx1[i];
                               double min=(xi*xi-1)/(xi*xi+1),max=1;
                               double escalaX=(double)radio;
                               double escalaY=(double)radio;
                               int x1=(int)(orgX+min*escalaX);int y1=(int)(orgY-reactancia(min)*escalaY);
                               int x2,y2;

                            for(double v=min; v<=max; v=v+entre){
                                  y2=orgY-(int)(reactancia(v)*escalaY);
                                  x2=orgX+(int)(v*escalaX);
                                  g2d.drawLine(x1, y1, x2, y2);
                                  x1=x2; y1=y2;
                            }


                               xi=-valorx1[i];
                              x1=(int)(orgX+min*escalaX);y1=(int)(orgY-reactancia2(min)*escalaY);
                              

                                 for(double v=min; v<=max; v=v+entre){
                                      y2=orgY-(int)(reactancia2(v)*escalaY);
                                      x2=orgX+(int)(v*escalaX);
                                      g2d.drawLine(x1, y1, x2, y2);
                                      x1=x2; y1=y2;
                                  }



                     }
    }

         private void DibujaPolar(Graphics2D g2d , double r, double thetai, double thetaf){

                      double escalaX=(double)radio;
                      double escalaY=(double)radio;
                      int y1=orgY-(int)(circulo(r,r*Math.cos(thetai),thetai)*escalaY);
                      int x1=orgX+(int)(r*Math.cos(thetai)*escalaX);
                      int x2,y2;
                  for(double j=thetai; j<=thetaf; j=j+0.001){
                          double v=r*Math.cos(j);
                          y2=orgY-(int)(circulo(r,v,j)*escalaY);
                          x2=orgX+(int)(v*escalaX);
                          g2d.drawLine(x1, y1, x2, y2);
                          x1=x2; y1=y2;
                  }



       }
                       private double circulo(double r, double v,double j){

                             if(Math.sin(j)>0) return Math.sqrt(r*r-v*v);
                             else return -Math.sqrt(r*r-v*v);
                       }



             private double resistencia(double x){
                     return java.lang.Math.sqrt(   Math.abs((1/(1+r))*(1/(1+r))-(x-r/(1+r))*(x-r/(1+r)))    );
             }
             
             private double resistencia(double x, double m){
                     return java.lang.Math.sqrt(   Math.abs((1/(1+m))*(1/(1+m))-(x-m/(1+m))*(x-m/(1+m)))    );
             }

             private double reactancia(double x){
                    return -Math.sqrt( Math.abs((1/xi)*(1/xi)-(x-1)*(x-1)) )+1/xi;
             }

             private double reactancia2(double x){
                   return Math.sqrt( Math.abs((1/xi)*(1/xi)-(x-1)*(x-1)) )+1/xi;
             }


            private double reactancia(double x,double m){
                  return -Math.sqrt( Math.abs((1/m)*(1/m)-(x-1)*(x-1)) )+1/m;
            }

            private double reactancia2(double x,double m){
                 return Math.sqrt( Math.abs((1/m)*(1/m)-(x-1)*(x-1)) )+1/m;
            }


         


            private double[] rValores(){
                double valor[]=new double[80];
                double delta=0.01;

                 for(int i=0;i<80;i++) valor[i]=0;
                 valor[0]=0.01;
                   for(int i=0;valor[i]<20;i++){
                        valor[i+1]=valor[i]+delta;

                        if(valor[i+1]>=0.2 &&  valor[i+1]<0.5) delta=0.02;    //0.02
                        else if(valor[i+1]>=0.5 &&  valor[i+1]<1) delta=0.05; //0.05
                        else if(valor[i+1]>=1 &&  valor[i+1]<2) delta=0.1;    //0.1
                        else if(valor[i+1]>=2 &&  valor[i+1]<5) delta=0.2;    //0.2
                        else if(valor[i+1]>=5 &&  valor[i+1]<10) delta=1;     //1
                        else if(valor[i+1]>=10 &&  valor[i+1]<=20) delta=2;   //2
                   }
                 return valor;
            }



                private double[] xValores1(){
                    double valor[]=new double[45];
                    double delta=0.01;

                     for(int i=0;i<45;i++) valor[i]=0;


                           valor[0]=0.01;
                           for(int i=0;valor[i]<1;i++){
                                valor[i+1]=valor[i]+delta;
                                if(valor[i+1]>=0.2 &&  valor[i+1]<0.5) delta=0.02;    //0.02
                                else if(valor[i+1]>=0.5 &&  valor[i+1]<1) delta=0.05; //0.05
                           }
                    return valor;
                 }


                 private double[] xValores2(){
                     double valor[]=new double[35];
                     double delta=0.1;

                    for(int i=0;i<35;i++) valor[i]=0;


                          valor[0]=1.1;
                               for(int i=0;valor[i]<20;i++){
                                    valor[i+1]=valor[i]+delta;

                                    if(valor[i+1]>=2 &&  valor[i+1]<5) delta=0.2;    //0.02
                                    else if(valor[i+1]>=5 &&  valor[i+1]<10) delta=1; //0.05
                                    else if(valor[i+1]>=10&&  valor[i+1]<20) delta=2;    //0.1
                               }
                     return valor;
                 }

                 private double CirculoRoev(){
                     
                          if(Math.abs(x1n)<=1){

                               double xii=(x1n*x1n+r1n*r1n-1)/(x1n*x1n+2*r1n+r1n*r1n+1);
                               double yii=resistencia(xii,r1n);
                               return Math.sqrt(xii*xii+yii*yii);
                            }
                          else   {
                               double x1naux;
                               if(x1n>0) x1naux=x1n;
                               else x1naux=-x1n;

                               double yii;
                               double xii=(x1naux*x1naux+r1n*r1n-1)/(x1naux*x1naux+2*r1n+r1n*r1n+1);

                               if(   Math.abs(resistencia(xii,r1n)-reactancia2(xii,x1naux) ) <0.005    )  yii=reactancia2(xii,x1naux);
                               else yii=reactancia(xii,x1naux);

                               return Math.sqrt(xii*xii+yii*yii);


                          }

                       
                }

                  /**
                   * Método privado que dibuja el círculo de ROEV cte. de la impedancia vista por la línea
                   * @param gd
                   */
                       private void marcacirculo(Graphics2D gd) {
                          double radium=CirculoRoev();
                          gd.drawOval((int)(orgX-radium*radio), (int)(orgY-radium*radio), (int)(2*radium*radio),(int) (2*radium*radio));


                       }

              
                 
          /**
           * Clase que define una Impedancia
           */
                  public class Impedancia{

                       public double R,X;

                               Impedancia(){
                                    R=0;
                                    X=0;


                                }

                               Impedancia(double re, double im){
                                   R=re;
                                   X=im;


                               }

                              public void serie(double re, double im){
                                     double a=re+R;
                                     double b=im+X;
                                     R=a;
                                     X=b;

                              }

                               public void serie(Impedancia z){
                                    double a=  z.R+R;
                                    double b= z.X+X;
                                    R=a;
                                    X=b;

                              }


                             public void paralelo(double re, double im){
                                    double a = ( (re + R )*(re*R - im*X) + (im+X)*(R*im+X*re))/( (re+R)*(re+R) + (im+X)*(im+X));
                                    double b=(-(R*re - im*X)*(im+X) + (R*im+X*re)*(re+R))/( (re+R)*(re+R) + (im+X)*(im+X));
                                    R=a;
                                    X=b;

                               }

                                public void paralelo(Impedancia z){
                                    double a=z.R*R-z.X*X;
                                    double b=R*z.X+X*z.R;
                                    R=a;
                                    X=b;

                               }

                  }

}
